]> Shamusworld >> Repos - rmac/blobdiff - sect.c
Fix for "*" getting bad section attributes, reported by A. Seed.
[rmac] / sect.c
diff --git a/sect.c b/sect.c
index d4277fcc0aad7f501d4b2b89d179945ab3b60033..7331cb3d765c4e867ce727cb39bce3b9e0c711d1 100644 (file)
--- a/sect.c
+++ b/sect.c
@@ -3,7 +3,7 @@
 // SECT.C - Code Generation, Fixups and Section Management
 // Copyright (C) 199x Landon Dyer, 2011 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
-// Source Utilised with the Kind Permission of Landon Dyer
+// Source utilised with the kind permission of Landon Dyer
 //
 
 #include "sect.h"
@@ -23,24 +23,24 @@ void MakeSection(int, WORD);
 void SwitchSection(int);
 
 // Section descriptors
-SECT sect[NSECTS];                             // All sections... 
-int cursect;                                   // Current section number
+SECT sect[NSECTS];             // All sections... 
+int cursect;                   // Current section number
 
 // These are copied from the section descriptor, the current code chunk
 // descriptor and the current fixup chunk descriptor when a switch is made into
 // a section.  They are copied back to the descriptors when the section is left.
-WORD scattr;                                   // Section attributes 
-LONG sloc;                                             // Current loc in section 
+WORD scattr;                   // Section attributes 
+LONG sloc;                             // Current loc in section 
 
-CHUNK * scode;                                 // Current (last) code chunk 
-LONG challoc;                                  // # bytes alloc'd to code chunk 
-LONG ch_size;                                  // # bytes used in code chunk 
-char * chptr;                                  // Deposit point in code chunk buffer 
+CHUNK * scode;                 // Current (last) code chunk 
+LONG challoc;                  // # bytes alloc'd to code chunk 
+LONG ch_size;                  // # bytes used in code chunk 
+char * chptr;                  // Deposit point in code chunk buffer 
 
-CHUNK * sfix;                                  // Current (last) fixup chunk
-LONG fchalloc;                                 // # bytes alloc'd to fixup chunk
-LONG fchsize;                                  // # bytes used in fixup chunk
-PTR fchptr;                                            // Deposit point in fixup chunk buffer
+CHUNK * sfix;                  // Current (last) fixup chunk
+LONG fchalloc;                 // # bytes alloc'd to fixup chunk
+LONG fchsize;                  // # bytes used in fixup chunk
+PTR fchptr;                            // Deposit point in fixup chunk buffer
 
 // Return a size (SIZB, SIZW, SIZL) or 0, depending on what kind of fixup is
 // associated with a location.
@@ -69,7 +69,7 @@ static char fusizoffs[] = {
 
 
 //
-// Initialize Sections; Setup initial ABS, TEXT, DATA and BSS sections
+// Initialize sections; setup initial ABS, TEXT, DATA and BSS sections
 //
 void InitSection(void)
 {
@@ -80,18 +80,19 @@ void InitSection(void)
                MakeSection(i, 0);
 
        // Construct default sections, make TEXT the current section
-       MakeSection(ABS,   SUSED | SABS | SBSS);                // ABS
-       MakeSection(TEXT,  SUSED | TEXT       );                // TEXT
-       MakeSection(DATA,  SUSED | DATA       );                // DATA
-       MakeSection(BSS,   SUSED | BSS  | SBSS);                // BSS
+       MakeSection(ABS,  SUSED | SABS | SBSS);         // ABS
+       MakeSection(TEXT, SUSED | TEXT       );         // TEXT
+       MakeSection(DATA, SUSED | DATA       );         // DATA
+       MakeSection(BSS,  SUSED | BSS  | SBSS);         // BSS
 //     MakeSection(M6502, SUSED | TEXT       );                // 6502 code section
 
-       SwitchSection(TEXT);                                            // Switch to TEXT for starters
+       // Switch to TEXT for starters
+       SwitchSection(TEXT);
 }
 
 
 //
-// Make a New (Clean) Section
+// Make a new (clean) section
 //
 void MakeSection(int sno, WORD attr)
 {
@@ -109,11 +110,12 @@ void MakeSection(int sno, WORD attr)
 //
 void SwitchSection(int sno)
 {
-       CHUNK * cp;                                                             // Chunk pointer
+       CHUNK * cp;
        cursect = sno;
        SECT * p = &sect[sno];
 
-       scattr = p->scattr;                                             // Copy section vars
+       // Copy section vars
+       scattr = p->scattr;
        sloc = p->sloc;
        scode = p->scode;
        sfix = p->sfix;
@@ -170,11 +172,12 @@ int fixtest(int sno, LONG loc)
        WORD w;
        LONG xloc;
 
-       stopmark();                                                             // Force update to sect[] variables
+       // Force update to sect[] variables
+       StopMark();
 
// Hairy, ugly linear search for a mark on our location;
// the speed doesn't matter, since this is only done when generating a
// listing, which is SLOW.
        for(ch=sect[sno].sffix; ch!=NULL; ch=ch->chnext)
        {
                fup.cp = (char *)ch->chptr;
@@ -195,7 +198,7 @@ int fixtest(int sno, LONG loc)
                                fup.lp += w;
                        }
                        else
-                               ++fup.lp;
+                               fup.lp++;
                }
        }
 
@@ -204,11 +207,11 @@ int fixtest(int sno, LONG loc)
 
 
 // 
-// Check that there are at least `amt' bytes left in the current chunk. If
-// there are not, allocate another chunk of at least `amt' bytes (and probably
+// Check that there are at least 'amt' bytes left in the current chunk. If
+// there are not, allocate another chunk of at least 'amt' bytes (and probably
 // more).
 // 
-// If `amt' is zero, ensure there are at least CH_THRESHOLD bytes, likewise.
+// If 'amt' is zero, ensure there are at least CH_THRESHOLD bytes, likewise.
 //
 int chcheck(LONG amt)
 {
@@ -257,8 +260,8 @@ int chcheck(LONG amt)
 }
 
 
-// This is really wrong. We need to make some proper structures here so we don't
-// have to count sizes of objects, that's what the compiler's for! :-P
+// This is really wrong. We need to make some proper structures here so we
+// don't have to count sizes of objects, that's what the compiler's for! :-P
 #define FIXUP_BASE_SIZE (sizeof(WORD) + sizeof(LONG) + sizeof(WORD) + sizeof(WORD))
 //
 // Arrange for a fixup on a location
@@ -269,7 +272,8 @@ int AddFixup(WORD attr, LONG loc, TOKEN * fexpr)
        LONG len = 0;
        CHUNK * cp;
        SECT * p;
-       // Shamus: Expression lengths are voodoo ATM (variable "i"). Need to fix this.
+       // Shamus: Expression lengths are voodoo ATM (variable "i"). Need to fix
+       //         this.
 #warning "!!! AddFixup() is filled with VOODOO !!!"
        DEBUG printf("FIXUP@$%X: $%X\n", loc, attr);
 
@@ -279,21 +283,25 @@ int AddFixup(WORD attr, LONG loc, TOKEN * fexpr)
        if (*fexpr == SYMBOL && fexpr[2] == ENDEXPR)
        {
                // Just a single symbol
-               // SCPCD : correct bit mask for attr (else other FU_xxx will match) NYAN !
+               // SCPCD : correct bit mask for attr (else other FU_xxx will match) 
+               // NYAN !
                if ((attr & FUMASKRISC) == FU_JR)
                {
+//printf("AddFixup: ((attr & FUMASKRISC) == FU_JR)\n");
 //                     i = 18;
 //                     i = FIXUP_BASE_SIZE + (sizeof(LONG) * 2);
                        i = FIXUP_BASE_SIZE + sizeof(SYM *) + sizeof(LONG);
                }
                else
                {
+//printf("AddFixup: ((attr & FUMASKRISC) == FU_JR) ELSE\n");
 //                     i = 14;
                        i = FIXUP_BASE_SIZE + sizeof(SYM *);
                }
        }
        else
        {
+//printf("AddFixup: !SYMBOL\n");
                attr |= FU_EXPR;
 
                for(len=0; fexpr[len]!=ENDEXPR; len++)
@@ -307,7 +315,7 @@ int AddFixup(WORD attr, LONG loc, TOKEN * fexpr)
                i = FIXUP_BASE_SIZE + sizeof(WORD) + (len * sizeof(TOKEN));
        }
 
-       // Maybe alloc another fixup chunk for this one to fit in
+       // Alloc another fixup chunk for this one to fit in if necessary
        if ((fchalloc - fchsize) < i)
        {
                p = &sect[cursect];
@@ -355,6 +363,7 @@ int AddFixup(WORD attr, LONG loc, TOKEN * fexpr)
        {
 //             *fchptr.lp++ = (LONG)fexpr[1];
                *fchptr.sy++ = symbolPtr[fexpr[1]];
+//printf("AddFixup: adding symbol (%s) [%08X]\n", symbolPtr[fexpr[1]]->sname, symbolPtr[fexpr[1]]->sattr);
        }
 
        // SCPCD : correct bit mask for attr (else other FU_xxx will match) NYAN !
@@ -372,7 +381,7 @@ int AddFixup(WORD attr, LONG loc, TOKEN * fexpr)
 
 
 //
-// Resolve all Fixups
+// Resolve all fixups
 //
 int ResolveAllFixups(void)
 {
@@ -381,7 +390,7 @@ int ResolveAllFixups(void)
 
        // Make undefined symbols GLOBL
        if (glob_flag)
-               syg_fix();
+               ForceUndefinedSymbolsGlobal();
 
        DEBUG printf("Resolving TEXT sections...\n");
        ResolveFixups(TEXT);
@@ -393,21 +402,21 @@ int ResolveAllFixups(void)
 
 
 //
-// Resolve Fixups in a Section
+// Resolve fixups in a section
 //
 int ResolveFixups(int sno)
 {
-       PTR fup;                                        // Current fixup
-       WORD * fuend;                           // End of last fixup (in this chunk)
-       WORD w;                                         // Fixup word (type+modes+flags)
-       char * locp;                            // Location to fix (in cached chunk) 
-       LONG loc;                                       // Location to fixup
-       VALUE eval;                                     // Expression value 
-       WORD eattr;                                     // Expression attrib
-       SYM * esym;                                     // External symbol involved in expr
-       SYM * sy;                                       // (Temp) pointer to a symbol
-       WORD i;                                         // (Temp) word
-       WORD tdb;                                       // eattr & TDB
+       PTR fup;                                // Current fixup
+       WORD * fuend;                   // End of last fixup (in this chunk)
+       WORD w;                                 // Fixup word (type+modes+flags)
+       char * locp;                    // Location to fix (in cached chunk) 
+       LONG loc;                               // Location to fixup
+       VALUE eval;                             // Expression value 
+       WORD eattr;                             // Expression attrib
+       SYM * esym;                             // External symbol involved in expr
+       SYM * sy;                               // (Temp) pointer to a symbol
+       WORD i;                                 // (Temp) word
+       WORD tdb;                               // eattr & TDB
        LONG oaddr;
        int reg2;
        WORD flags;
@@ -425,7 +434,7 @@ int ResolveFixups(int sno)
        // "Cache" first chunk
        CHUNK * cch = sc->sfcode;
 
-       // Can't fixup a sect with nothing in it
+       // Can't fixup a section with nothing in it
        if (cch == NULL)
                return 0;
 
@@ -890,7 +899,8 @@ DEBUG { printf("ResolveFixups: cfileno=%u\n", cfileno); }
                                break;
 
                        default:
-                               interror(4);                                 // Bad fixup type
+                               // Bad fixup type--this should *never* happen!
+                               interror(4);
                                // NOTREACHED
                        }
                        continue;
@@ -170,11 +172,12 @@ int fixtest(int sno, LONG loc)
 
-       // Hairy, ugly linear search for a mark on our location;
-       // the speed doesn't matter, since this is only done when generating a
-       // listing, which is SLOW.
+       // Hairy, ugly linear search for a mark on our location; the speed doesn't
+       // matter, since this is only done when generating a listing, which is
+       // SLOW.
        for(ch=sect[sno].sffix; ch!=NULL; ch=ch->chnext)
        {